@@ -8,10 +8,10 @@ import datetime
 import enum
 import sys
 
-from typing import NamedTuple, Union, Optional, List, Set, Dict, Tuple, Any
+from decimal import Decimal
+from typing import Any, Dict, List, NamedTuple, Optional, Set, Tuple, Union
 
 from beancount.core.amount import Amount
-from beancount.core.number import Decimal
 from beancount.core.number import D
 from beancount.core.position import Cost
 from beancount.core.position import CostSpec
@@ -20,7 +20,6 @@ from beancount.utils.bisect_key import bisect_left_with_key
 
 
 # Type declarations.
-# pylint: disable=invalid-name
 Account = str
 Currency = str
 Flag = str
@@ -172,12 +171,6 @@ Balance = new_directive('Balance', [
 # entry field should be set to.
 #
 # Attributes:
-#   entry: A Transaction instance (see above), which the posting applies to.
-#     It is convenient to have Posting instances point to their parent entries,
-#     because account journals contain lists of Postings and non-Transaction
-#     entries and though it creates a circular dependency between Transaction
-#     and Posting, it allows us to easily resolve the lists of Postings to their
-#     transactions for rendering.
 #   account: A string, the account that is modified by this posting.
 #   units: An Amount, the units of the position.
 #   cost: A Cost or CostSpec instances, the units of the position.
@@ -218,7 +211,7 @@ Posting = NamedTuple('Posting', [
 #   tags: A set of tag strings (without the '#'), or EMPTY_SET.
 #   links: A set of link strings (without the '^'), or EMPTY_SET.
 #   postings: A list of Posting instances, the legs of this transaction. See the
-#     doc under Posting below.
+#     doc under Posting above.
 Transaction = new_directive('Transaction', [
     ('flag', Flag),
     ('payee', Optional[str]),
@@ -402,8 +395,9 @@ Directive = Union[
     Custom
 ]
 
-# Type for the list of entries.
+# Type for the list of entries and options map.
 Entries = List[Directive]
+Options = Dict[str, Any]
 
 
 def new_metadata(filename, lineno, kvlist=None):
@@ -541,11 +535,11 @@ def transaction_has_conversion(transaction):
     Args:
       transaction: an instance of a Transaction entry.
     Returns:
-      A boolean, true if this transacation contains at least one posting with a
+      A boolean, true if this transaction contains at least one posting with a
       price conversion.
     """
     assert isinstance(transaction, Transaction), (
-        "Invalid type of entry for Transaction: {}".format(transaction))
+        "Invalid type of entry for transaction: {}".format(transaction))
     for posting in transaction.postings:
         if posting_has_conversion(posting):
             return True
@@ -641,10 +635,10 @@ def has_entry_account_component(entry, component):
     Args:
       entry: A Transaction entry.
       component: A string, a component of an account name. For instance,
-        'Food' in 'Expenses:Food:Restaurant'. All components are considered.
+        ``Food`` in ``Expenses:Food:Restaurant``. All components are considered.
     Returns:
-      A boolean, true if the component is in the account. Note that a component
-      name must be whole, that is 'NY' is not in Expenses:Taxes:StateNY'.
+      Boolean: true if the component is in the account. Note that a component
+      name must be whole, that is ``NY`` is not in ``Expenses:Taxes:StateNY``.
     """
     return (isinstance(entry, Transaction) and
             any(has_component(posting.account, component)
@@ -671,7 +665,7 @@ def find_closest(entries, filename, lineno):
         emeta = entry.meta
         if emeta["filename"] == filename and emeta["lineno"] > 0:
             diffline = lineno - emeta["lineno"]
-            if diffline >= 0 and diffline < min_diffline:
+            if 0 <= diffline < min_diffline:
                 min_diffline = diffline
                 closest_entry = entry
     return closest_entry
