@@ -5,16 +5,15 @@ __license__ = "GNU GPLv2"
 
 import collections
 import copy
-import os
 import re
 import sys
 import traceback
 from os import path
 from datetime import date
+from decimal import Decimal
 
 from beancount.core.number import ZERO
 from beancount.core.number import MISSING
-from beancount.core.number import Decimal
 from beancount.core.amount import Amount
 from beancount.core import display_context
 from beancount.core.position import CostSpec
@@ -41,14 +40,6 @@ from beancount.core import account
 from beancount.core import data
 
 
-# FIXME: This environment variable enables temporary support for negative
-# prices. If you've updated across 2015-01-10 and you're getting a lot of
-# errors, you need to fix all the signs on your @@ total price values (they are
-# to be positive only). Just set this environment variable to disable this
-# change if you need to post-pone this.
-__allow_negative_prices__ = os.environ.get('BEANCOUNT_ALLOW_NEGATIVE_PRICES', False)
-
-
 ParserError = collections.namedtuple('ParserError', 'source message entry')
 ParserSyntaxError = collections.namedtuple('ParserSyntaxError', 'source message entry')
 DeprecatedError = collections.namedtuple('DeprecatedError', 'source message entry')
@@ -121,7 +112,8 @@ class Builder(lexer.LexBuilder):
     """A builder used by the lexer and grammar parser as callbacks to create
     the data objects corresponding to rules parsed from the input file."""
 
-    def __init__(self, filename):
+    # pylint: disable=too-many-instance-attributes
+    def __init__(self):
         lexer.LexBuilder.__init__(self)
 
         # A stack of the current active tags.
@@ -136,8 +128,8 @@ class Builder(lexer.LexBuilder):
         # Accumulated and unprocessed options.
         self.options = copy.deepcopy(options.OPTIONS_DEFAULTS)
 
-        # Set the filename we're processing.
-        self.options['filename'] = filename
+        # A mapping of all the accounts created.
+        self.accounts = {}
 
         # Make the account regexp more restrictive than the default: check
         # types. Warning: This overrides the value in the base class.
@@ -145,12 +137,12 @@ class Builder(lexer.LexBuilder):
 
         # A display context builder.
         self.dcontext = display_context.DisplayContext()
-        self._dcupdate = self.dcontext.update
+        self.display_context_update = self.dcontext.update
 
-    def dcupdate(self, number, currency):
+    def _dcupdate(self, number, currency):
         """Update the display context."""
         if isinstance(number, Decimal) and currency and currency is not MISSING:
-            self._dcupdate(number, currency)
+            self.display_context_update(number, currency)
 
     def finalize(self):
         """Finalize the parser, check for final errors and return the triple.
@@ -198,29 +190,13 @@ class Builder(lexer.LexBuilder):
         # Build and store the inferred DisplayContext instance.
         self.options['dcontext'] = self.dcontext
 
-        # Add the full list of seen commodities.
-        #
-        # IMPORTANT: This is currently where the list of all commodities seen
-        # from the parser lives. The
-        # beancount.core.getters.get_commodities_map() routine uses this to
-        # automatically generate a full list of directives. An alternative would
-        # be to implement a plugin that enforces the generate of these
-        # post-parsing so that they are always guaranteed to live within the
-        # flow of entries. This would allow us to keep all the data in that list
-        # of entries and to avoid depending on the options to store that output.
-        self.options['commodities'] = self.commodities
-
         return self.options
 
-    def get_invalid_account(self):
-        """See base class."""
-        return account.join(self.options['name_equity'], 'InvalidAccountName')
-
     def get_long_string_maxlines(self):
         """See base class."""
         return self.options['long_string_maxlines']
 
-    def store_result(self, entries):
+    def store_result(self, filename, lineno, entries):
         """Start rule stores the final result here.
 
         Args:
@@ -228,6 +204,8 @@ class Builder(lexer.LexBuilder):
         """
         if entries:
             self.entries = entries
+        # Also record the name of the processed file.
+        self.options['filename'] = filename
 
     def build_grammar_error(self, filename, lineno, exc_value,
                             exc_type=None, exc_traceback=None):
@@ -252,6 +230,22 @@ class Builder(lexer.LexBuilder):
         self.errors.append(
             ParserSyntaxError(meta, message, None))
 
+    def account(self, filename, lineno, account):
+        """Check account name validity.
+
+        Args:
+          account: a str, the account name.
+        Returns:
+          A string, the account name.
+        """
+        if not self.account_regexp.match(account):
+            meta = new_metadata(filename, lineno)
+            self.errors.append(
+                ParserError(meta, "Invalid account name: {}".format(account), None))
+        # Intern account names. This should reduces memory usage a
+        # fair bit because these strings are repeated liberally.
+        return self.accounts.setdefault(account, account)
+
     def pipe_deprecated_error(self, filename, lineno):
         """Issue a 'Pipe deprecated' error.
 
@@ -265,7 +259,7 @@ class Builder(lexer.LexBuilder):
         self.errors.append(
             ParserSyntaxError(meta, "Pipe symbol is deprecated.", None))
 
-    def pushtag(self, tag):
+    def pushtag(self, filename, lineno, tag):
         """Push a tag on the current set of tags.
 
         Note that this does not need to be stack ordered.
@@ -275,7 +269,7 @@ class Builder(lexer.LexBuilder):
         """
         self.tags.append(tag)
 
-    def poptag(self, tag):
+    def poptag(self, filename, lineno, tag):
         """Pop a tag off the current set of stacks.
 
         Args:
@@ -284,19 +278,20 @@ class Builder(lexer.LexBuilder):
         try:
             self.tags.remove(tag)
         except ValueError:
-            meta = new_metadata(self.options['filename'], 0)
+            meta = new_metadata(filename, lineno)
             self.errors.append(
                 ParserError(meta, "Attempting to pop absent tag: '{}'".format(tag), None))
 
-    def pushmeta(self, key, value):
+    def pushmeta(self, filename, lineno, key_value):
         """Set a metadata field on the current key-value pairs to be added to transactions.
 
         Args:
           key_value: A KeyValue instance, to be added to the dict of metadata.
         """
+        key, value = key_value
         self.meta[key].append(value)
 
-    def popmeta(self, key):
+    def popmeta(self, filename, lineno, key):
         """Removed a key off the current set of stacks.
 
         Args:
@@ -310,7 +305,7 @@ class Builder(lexer.LexBuilder):
             if not value_list:
                 self.meta.pop(key)
         except IndexError:
-            meta = new_metadata(self.options['filename'], 0)
+            meta = new_metadata(filename, lineno)
             self.errors.append(
                 ParserError(meta,
                             "Attempting to pop absent metadata key: '{}'".format(key),
@@ -418,7 +413,7 @@ class Builder(lexer.LexBuilder):
         """
         self.options['plugin'].append((plugin_name, plugin_config))
 
-    def amount(self, number, currency):
+    def amount(self, filename, lineno, number, currency):
         """Process an amount grammar rule.
 
         Args:
@@ -429,10 +424,10 @@ class Builder(lexer.LexBuilder):
         """
         # Update the mapping that stores the parsed precisions.
         # Note: This is relatively slow, adds about 70ms because of number.as_tuple().
-        self.dcupdate(number, currency)
+        self._dcupdate(number, currency)
         return Amount(number, currency)
 
-    def compound_amount(self, number_per, number_total, currency):
+    def compound_amount(self, filename, lineno, number_per, number_total, currency):
         """Process an amount grammar rule.
 
         Args:
@@ -445,18 +440,18 @@ class Builder(lexer.LexBuilder):
         """
         # Update the mapping that stores the parsed precisions.
         # Note: This is relatively slow, adds about 70ms because of number.as_tuple().
-        self.dcupdate(number_per, currency)
-        self.dcupdate(number_total, currency)
+        self._dcupdate(number_per, currency)
+        self._dcupdate(number_total, currency)
 
         # Note that we are not able to reduce the value to a number per-share
         # here because we only get the number of units in the full lot spec.
         return CompoundAmount(number_per, number_total, currency)
 
-    def cost_merge(self, _):
+    def cost_merge(self, filename, lineno, _):
         """Create a 'merge cost' token."""
         return MERGE_COST
 
-    def cost_spec(self, cost_comp_list, is_total):
+    def cost_spec(self, filename, lineno, cost_comp_list, is_total):
         """Process a cost_spec grammar rule.
 
         Args:
@@ -484,7 +479,7 @@ class Builder(lexer.LexBuilder):
                     compound_cost = comp
                 else:
                     self.errors.append(
-                        ParserError(self.get_lexer_location(),
+                        ParserError(new_metadata(filename, lineno),
                                     "Duplicate cost: '{}'.".format(comp), None))
 
             elif isinstance(comp, date):
@@ -492,18 +487,18 @@ class Builder(lexer.LexBuilder):
                     date_ = comp
                 else:
                     self.errors.append(
-                        ParserError(self.get_lexer_location(),
+                        ParserError(new_metadata(filename, lineno),
                                     "Duplicate date: '{}'.".format(comp), None))
 
             elif comp is MERGE_COST:
                 if merge is None:
                     merge = True
                     self.errors.append(
-                        ParserError(self.get_lexer_location(),
+                        ParserError(new_metadata(filename, lineno),
                                     "Cost merging is not supported yet", None))
                 else:
                     self.errors.append(
-                        ParserError(self.get_lexer_location(),
+                        ParserError(new_metadata(filename, lineno),
                                     "Duplicate merge-cost spec", None))
 
             else:
@@ -513,7 +508,7 @@ class Builder(lexer.LexBuilder):
                     label = comp
                 else:
                     self.errors.append(
-                        ParserError(self.get_lexer_location(),
+                        ParserError(new_metadata(filename, lineno),
                                     "Duplicate label: '{}'.".format(comp), None))
 
         # If there was a cost_comp_list, thus a "{...}" cost basis spec, you must
@@ -527,7 +522,7 @@ class Builder(lexer.LexBuilder):
                 if number_total is not None:
                     self.errors.append(
                         ParserError(
-                            self.get_lexer_location(),
+                            new_metadata(filename, lineno),
                             ("Per-unit cost may not be specified using total cost "
                              "syntax: '{}'; ignoring per-unit cost").format(compound_cost),
                             None))
@@ -543,7 +538,7 @@ class Builder(lexer.LexBuilder):
 
         return CostSpec(number_per, number_total, currency, date_, label, merge)
 
-    def handle_list(self, object_list, new_object):
+    def handle_list(self, filename, lineno, object_list, new_object):
         """Handle a recursive list grammar rule, generically.
 
         Args:
@@ -744,7 +739,7 @@ class Builder(lexer.LexBuilder):
         if not path.isabs(document_filename):
             document_filename = path.abspath(path.join(path.dirname(filename),
                                                        document_filename))
-        tags, links = self.finalize_tags_links(tags_links.tags, tags_links.links)
+        tags, links = self._finalize_tags_links(tags_links.tags, tags_links.links)
         return Document(meta, date, account, document_filename, tags, links)
 
     def custom(self, filename, lineno, date, dir_type, custom_values, kvlist):
@@ -763,7 +758,7 @@ class Builder(lexer.LexBuilder):
         meta = new_metadata(filename, lineno, kvlist)
         return Custom(meta, date, dir_type, custom_values)
 
-    def custom_value(self, value, dtype=None):
+    def custom_value(self, filename, lineno, value, dtype=None):
         """Create a custom value object, along with its type.
 
         Args:
@@ -776,7 +771,7 @@ class Builder(lexer.LexBuilder):
             dtype = type(value)
         return ValueType(value, dtype)
 
-    def key_value(self, key, value):
+    def key_value(self, filename, lineno, key, value):
         """Process a document directive.
 
         Args:
@@ -809,16 +804,15 @@ class Builder(lexer.LexBuilder):
         meta = new_metadata(filename, lineno)
 
         # Prices may not be negative.
-        if not __allow_negative_prices__:
-            if price and isinstance(price.number, Decimal) and price.number < ZERO:
-                self.errors.append(
-                    ParserError(meta, (
-                        "Negative prices are not allowed: {} "
-                        "(see http://furius.ca/beancount/doc/bug-negative-prices "
-                        "for workaround)"
-                    ).format(price), None))
-                # Fix it and continue.
-                price = Amount(abs(price.number), price.currency)
+        if price and isinstance(price.number, Decimal) and price.number < ZERO:
+            self.errors.append(
+                ParserError(meta, (
+                    "Negative prices are not allowed: {} "
+                    "(see http://furius.ca/beancount/doc/bug-negative-prices "
+                    "for workaround)"
+                ).format(price), None))
+            # Fix it and continue.
+            price = Amount(abs(price.number), price.currency)
 
         # If the price is specified for the entire amount, compute the effective
         # price here and forget about that detail of the input syntax.
@@ -826,10 +820,9 @@ class Builder(lexer.LexBuilder):
             if units.number == ZERO:
                 number = ZERO
             else:
-                if __allow_negative_prices__:
-                    number = price.number/units.number
-                else:
-                    number = price.number/abs(units.number)
+                number = price.number
+                if number is not MISSING:
+                    number = number/abs(units.number)
             price = Amount(number, price.currency)
 
         # Note: Allow zero prices because we need them for round-trips for
@@ -853,7 +846,7 @@ class Builder(lexer.LexBuilder):
 
         return Posting(account, units, cost, price, chr(flag) if flag else None, meta)
 
-    def tag_link_new(self, _):
+    def tag_link_new(self, filename, lineno):
         """Create a new TagsLinks instance.
 
         Returns:
@@ -861,7 +854,7 @@ class Builder(lexer.LexBuilder):
         """
         return TagsLinks(set(), set())
 
-    def tag_link_TAG(self, tags_links, tag):
+    def tag_link_TAG(self, filename, lineno, tags_links, tag):
         """Add a tag to the TagsLinks accumulator.
 
         Args:
@@ -873,7 +866,7 @@ class Builder(lexer.LexBuilder):
         tags_links.tags.add(tag)
         return tags_links
 
-    def tag_link_LINK(self, tags_links, link):
+    def tag_link_LINK(self, filename, lineno, tags_links, link):
         """Add a link to the TagsLinks accumulator.
 
         Args:
@@ -885,19 +878,7 @@ class Builder(lexer.LexBuilder):
         tags_links.links.add(link)
         return tags_links
 
-    def tag_link_STRING(self, tags_links, string):
-        """Add a string to the TagsLinks accumulator.
-
-        Args:
-          tags_links: The current TagsLinks accumulator.
-          string: A string, the new string to insert in the list.
-        Returns:
-          An updated TagsLinks instance.
-        """
-        tags_links.strings.append(string)
-        return tags_links
-
-    def unpack_txn_strings(self, txn_strings, meta):
+    def _unpack_txn_strings(self, txn_strings, meta):
         """Unpack a tags_links accumulator to its payee and narration fields.
 
         Args:
@@ -922,7 +903,7 @@ class Builder(lexer.LexBuilder):
             return None
         return payee, narration
 
-    def finalize_tags_links(self, tags, links):
+    def _finalize_tags_links(self, tags, links):
         """Finally amend tags and links and return final objects to be inserted.
 
         Args:
@@ -975,8 +956,9 @@ class Builder(lexer.LexBuilder):
                 elif isinstance(posting_or_kv, TagsLinks):
                     if postings:
                         self.errors.append(ParserError(
-                            meta, "Tags or links not allowed after first Posting".format(
-                                posting_or_kv), None))
+                            meta,
+                            "Tags or links not allowed after first " +
+                            "Posting: {}".format(posting_or_kv), None))
                     else:
                         tags.update(posting_or_kv.tags)
                         links.update(posting_or_kv.links)
@@ -1002,7 +984,7 @@ class Builder(lexer.LexBuilder):
                                     posting_or_kv), None))
 
         # Freeze the tags & links or set to default empty values.
-        tags, links = self.finalize_tags_links(tags, links)
+        tags, links = self._finalize_tags_links(tags, links)
 
         # Initialize the metadata fields from the set of active values.
         if self.meta:
@@ -1014,7 +996,7 @@ class Builder(lexer.LexBuilder):
             meta.update(explicit_meta)
 
         # Unpack the transaction fields.
-        payee_narration = self.unpack_txn_strings(txn_strings, meta)
+        payee_narration = self._unpack_txn_strings(txn_strings, meta)
         if payee_narration is None:
             return None
         payee, narration = payee_narration
