@@ -9,21 +9,21 @@ import unittest
 import inspect
 import textwrap
 import re
+from decimal import Decimal
 from unittest import mock
 
 from beancount.core.number import D
-from beancount.core.number import Decimal
 from beancount.core.number import ZERO
 from beancount.core.number import MISSING
 from beancount.core.amount import from_string as A
 from beancount.core.amount import Amount
 from beancount.core.position import CostSpec
+from beancount.parser import grammar
 from beancount.parser import parser
 from beancount.parser import lexer
 from beancount.core import data
 from beancount.core import amount
 from beancount.utils import test_utils
-from beancount.parser import grammar
 from beancount.parser import cmptest
 
 
@@ -228,6 +228,29 @@ class TestParserEntryTypes(unittest.TestCase):
                          txns[0].values)
 
 
+class TestWhitespace(unittest.TestCase):
+    """Tests for handling of whitespace and indent."""
+
+    @parser.parse_doc(expect_errors=True)
+    def test_indent_error_0(self, entries, errors, _):
+        """
+          2020-07-28 open Assets:Foo
+            2020-07-28 open Assets:Bar
+        """
+        self.assertEqual(len(errors), 1)
+        self.assertRegex(errors[0].message, "unexpected DATE")
+
+    @parser.parse_doc(expect_errors=True)
+    def test_indent_error_1(self, entries, errors, _):
+        """
+          2020-07-28 open Assets:Foo
+
+            2020-07-28 open Assets:Bar
+        """
+        self.assertEqual(len(errors), 1)
+        self.assertRegex(errors[0].message, "unexpected INDENT")
+
+
 class TestParserComplete(unittest.TestCase):
     """Tests of completion of balance."""
 
@@ -279,9 +302,12 @@ class TestUglyBugs(unittest.TestCase):
         check_list(self, entries, [])
         check_list(self, errors, [])
 
-    def test_extra_whitespace_note(self):
-        input_ = '\n2013-07-11 note Assets:Cash "test"\n\n  ;;\n'
-        entries, errors, _ = parser.parse_string(input_)
+    @parser.parse_doc()
+    def test_extra_whitespace_note(self, entries, errors, _):
+        """
+        2013-07-11 note Assets:Cash "test"
+          ;;
+        """
         check_list(self, entries, [data.Note])
         check_list(self, errors, [])
 
@@ -294,7 +320,7 @@ class TestUglyBugs(unittest.TestCase):
             ';; End of file',
         ])
 
-        entries, errors, _ = parser.parse_string(input_, yydebug=0)
+        entries, errors, _ = parser.parse_string(input_)
         check_list(self, entries, [data.Transaction])
         check_list(self, errors, [])
 
@@ -327,7 +353,7 @@ class TestUglyBugs(unittest.TestCase):
         """
           2013-05-01 open Assets:Cash   USD,CAD,EUR
           2013-05-02 close Assets:US:BestBank:Checking
-          2013-05-03 pad Assets:US:BestBank:Checking  Equity:Opening-Balancess
+          2013-05-03 pad Assets:US:BestBank:Checking  Equity:Opening-Balances
           2013-05-04 event "location" "New York, USA"
           2013-05-05 * "Payee" "Narration"
             Assets:US:BestBank:Checking   100.00 USD
@@ -483,7 +509,7 @@ class TestPushPopMeta(unittest.TestCase):
         """
         self.assertEqual(1, len(errors))
         self.assertRegex(errors[0].message,
-                                 "Attempting to pop absent metadata key")
+                         "Attempting to pop absent metadata key")
 
     @parser.parse_doc(expect_errors=True)
     def test_pushmeta_forgotten(self, entries, errors, _):
@@ -492,7 +518,7 @@ class TestPushPopMeta(unittest.TestCase):
         """
         self.assertEqual(1, len(errors))
         self.assertRegex(errors[0].message,
-                                 "Unbalanced metadata key")
+                         "Unbalanced metadata key")
 
 
 class TestMultipleLines(unittest.TestCase):
@@ -559,30 +585,6 @@ class TestSyntaxErrors(unittest.TestCase):
         self.assertEqual(2, len(entries[0].postings))
 
 
-class TestLineNumbers(unittest.TestCase):
-    """Check that the line numbers line up correctly."""
-
-    @parser.parse_doc()
-    def test_line_numbers(self, entries, errors, _):
-        """
-          2013-05-18 * "Nice dinner at Mermaid Inn"
-            Expenses:Restaurant         100 USD
-            Assets:US:Cash             -100 USD
-
-          2013-05-19 balance  Assets:US:Cash   -100 USD
-
-          2013-05-20 note  Assets:US:Cash   "Something"
-
-        """
-        _, first_line = inspect.getsourcelines(
-            TestLineNumbers.test_line_numbers.__wrapped__)
-        first_line += 1
-
-        self.assertEqual(2, entries[0].meta["lineno"] - first_line)
-        self.assertEqual(6, entries[1].meta["lineno"] - first_line)
-        self.assertEqual(8, entries[2].meta["lineno"] - first_line)
-
-
 class TestParserOptions(unittest.TestCase):
 
     @parser.parse_doc()
@@ -653,9 +655,7 @@ class TestParserInclude(unittest.TestCase):
                          options_map['include'])
 
     def test_include_relative_from_string(self):
-        input_string = """
-          include "some/relative/filename.beancount"
-        """
+        input_string = 'include "some/relative/filename.beancount"'
         entries, errors, options_map = parser.parse_string(input_string)
         self.assertFalse(errors)
         self.assertEqual(['some/relative/filename.beancount'],
@@ -1254,7 +1254,7 @@ class TestTotalsAndSigns(unittest.TestCase):
             Assets:Investments:MSFT      0 MSFT {200.00 USD}
             Assets:Investments:Cash      0 USD
         """
-        pass # Should produce no errors.
+        # Should produce no errors.
 
     @parser.parse_doc(expect_errors=False)
     def test_zero_cost(self, entries, errors, _):
@@ -1263,7 +1263,7 @@ class TestTotalsAndSigns(unittest.TestCase):
             Assets:Investments:MSFT      -10 MSFT {0.00 USD}
             Assets:Investments:Cash     0.00 USD
         """
-        pass # Should produce no errors.
+        # Should produce no errors.
 
     @parser.parse_doc(expect_errors=False)
     def test_cost_negative(self, entries, errors, _):
@@ -1272,7 +1272,7 @@ class TestTotalsAndSigns(unittest.TestCase):
             Assets:Investments:MSFT      -10 MSFT {-200.00 USD}
             Assets:Investments:Cash  2000.00 USD
         """
-        pass # Should produce no errors.
+        # Should produce no errors.
         # Note: This error is caught only at booking time.
 
     @parser.parse_doc()
@@ -1320,7 +1320,7 @@ class TestTotalsAndSigns(unittest.TestCase):
             Assets:Investments:MSFT      -10 MSFT {{-200.00 USD}}
             Assets:Investments:Cash   200.00 USD
         """
-        pass # Should produce no errors.
+        # Should produce no errors.
         # Note: This error is caught only at booking time.
 
     @parser.parse_doc(expect_errors=True)
@@ -1364,71 +1364,6 @@ class TestTotalsAndSigns(unittest.TestCase):
         self.assertRegex(errors[0].message, 'Negative.*allowed')
 
 
-class TestAllowNegativePrices(unittest.TestCase):
-
-    def setUp(self):
-        self.__allow_negative_prices__ = grammar.__allow_negative_prices__
-        grammar.__allow_negative_prices__ = True
-
-    def tearDown(self):
-        grammar.__allow_negative_prices__ = self.__allow_negative_prices__
-
-    @parser.parse_doc()
-    def test_total_cost(self, entries, errors, _):
-        """
-          2013-05-18 * ""
-            Assets:Investments:MSFT      10 MSFT {{2,000 USD}}
-            Assets:Investments:Cash  -20000 USD
-
-          2013-05-18 * ""
-            Assets:Investments:MSFT      10 MSFT {{2000 USD, 2014-02-25}}
-            Assets:Investments:Cash  -20000 USD
-
-          2013-06-01 * ""
-            Assets:Investments:MSFT      -10 MSFT {{2,000 USD}}
-            Assets:Investments:Cash    20000 USD
-        """
-        self.assertFalse(errors)
-        for entry in entries:
-            posting = entry.postings[0]
-            self.assertEqual(D('2000'), posting.cost.number_total)
-            self.assertEqual('USD', posting.cost.currency)
-            self.assertEqual(None, posting.price)
-
-    @parser.parse_doc()
-    def test_price_negative(self, entries, errors, _):
-        """
-          2013-05-18 * ""
-            Assets:Investments:MSFT      -10 MSFT @ -200.00 USD
-            Assets:Investments:Cash  2000.00 USD
-        """
-        posting = entries[0].postings[0]
-        self.assertEqual(amount.from_string('-200 USD'), posting.price)
-        self.assertEqual(None, posting.cost)
-
-    @parser.parse_doc()
-    def test_total_price_negative(self, entries, errors, _):
-        """
-          2013-05-18 * ""
-            Assets:Investments:MSFT        -10 MSFT @@ 2000.00 USD
-            Assets:Investments:Cash   20000.00 USD
-        """
-        posting = entries[0].postings[0]
-        self.assertEqual(amount.from_string('-200 USD'), posting.price)
-        self.assertEqual(None, posting.cost)
-
-    @parser.parse_doc()
-    def test_total_price_inverted(self, entries, errors, _):
-        """
-          2013-05-18 * ""
-            Assets:Investments:MSFT         10 MSFT @@ -2000.00 USD
-            Assets:Investments:Cash  -20000.00 USD
-        """
-        posting = entries[0].postings[0]
-        self.assertEqual(amount.from_string('-200 USD'), posting.price)
-        self.assertEqual(None, posting.cost)
-
-
 class TestBalance(unittest.TestCase):
 
     @parser.parse_doc()
@@ -1838,7 +1773,7 @@ class TestLexerAndParserErrors(cmptest.TestCase):
       builder method is invoked and raises a Python exception. A macro in the
       code that invokes this method is used to catch this error and calls
       build_grammar_error_from_exception() to register an error and makes the
-      parser issue an error wth YYERROR (see {05bb0fb60e86}).
+      parser issue an error with YYERROR (see {05bb0fb60e86}).
 
     We never call YYABORT anywhere so the yyparse() function should never return
     anything else than 0. If it does, we translate that into a Python
@@ -1937,7 +1872,7 @@ class TestLexerAndParserErrors(cmptest.TestCase):
             self.assertEqual(1, len(entries))
             self.assertEqual(1, len(errors))
             self.assertRegex(errors[0].message,
-                                     '(Invalid token|unexpected RPAREN)')
+                             '(Invalid token|unexpected RPAREN)')
 
     @parser.parse_doc(expect_errors=True)
     def test_grammar_syntax_error(self, entries, errors, _):
@@ -2563,8 +2498,6 @@ class TestMisc(cmptest.TestCase):
         """
         self.assertEqual(0, len(errors))
 
-    # FIXME(blais): Ideally this unindented should generate an error.
-    # It would be nicer if only indented comments would be allowed.
     @parser.parse_doc(expect_errors=True)
     def test_comment_in_postings_invalid(self, entries, errors, options_map):
         """
@@ -2574,7 +2507,7 @@ class TestMisc(cmptest.TestCase):
             Assets:Crypto:Bitcoin                -0.00082487 BTC
         """
         self.assertEqual(1, len(errors))
-        self.assertRegex(errors[0].message, "unexpected ACCOUNT")
+        self.assertRegex(errors[0].message, "unexpected INDENT")
 
 
 class TestDocument(unittest.TestCase):
@@ -2603,3 +2536,22 @@ class TestDocument(unittest.TestCase):
         """
         check_list(self, entries, [data.Document])
         self.assertEqual({'something'}, entries[0].links)
+
+
+class TestMethodsSignature(unittest.TestCase):
+
+    def test_signatures(self):
+        # Enforce that all "public" methods of the Builder class have
+        # 'filename' and 'lineno' as the first two arguments.
+        for name, func in inspect.getmembers(grammar.Builder, inspect.isfunction):
+            if (name.isupper() or
+                name.startswith('_') or
+                name.startswith('get_') or
+                name == 'finalize'):
+                continue
+            parameters = inspect.signature(func).parameters.keys()
+            self.assertEqual(list(parameters)[:3], ['self', 'filename', 'lineno'])
+
+
+if __name__ == '__main__':
+    unittest.main()
