@@ -9,8 +9,8 @@ import io
 import re
 import sys
 import textwrap
+from decimal import Decimal
 
-from beancount.core.number import Decimal
 from beancount.core import position
 from beancount.core import convert
 from beancount.core import inventory
@@ -88,21 +88,27 @@ def align_position_strings(strings):
 class EntryPrinter:
     """A multi-method interface for printing all directive types.
 
-    Atributes:
+    Attributes:
       dcontext: An instance of DisplayContext with which to render all the numbers.
       render_weight: A boolean, true if we should render the weight of the postings
         as a comment, for debugging.
       min_width_account: An integer, the minimum width to leave for the account name.
+      prefix: User-specific prefix for custom indentation (for Fava).
+      stringify_invalid_types: If a metadata value is invalid, force a conversion to
+        string for printout.
     """
 
     # pylint: disable=invalid-name
 
-    def __init__(self, dcontext=None, render_weight=False, min_width_account=None):
+    def __init__(self, dcontext=None, render_weight=False, min_width_account=None,
+                 prefix=None, stringify_invalid_types=False):
         self.dcontext = dcontext or display_context.DEFAULT_DISPLAY_CONTEXT
         self.dformat = self.dcontext.build(precision=display_context.Precision.MOST_COMMON)
         self.dformat_max = self.dcontext.build(precision=display_context.Precision.MAXIMUM)
         self.render_weight = render_weight
         self.min_width_account = min_width_account
+        self.prefix = prefix or '  '
+        self.stringify_invalid_types = stringify_invalid_types
 
     def __call__(self, obj):
         """Render a directive.
@@ -119,7 +125,7 @@ class EntryPrinter:
 
     META_IGNORE = set(['filename', 'lineno', '__automatic__'])
 
-    def write_metadata(self, meta, oss, prefix='  '):
+    def write_metadata(self, meta, oss, prefix=None):
         """Write metadata to the file object, excluding filename and line number.
 
         Args:
@@ -128,6 +134,8 @@ class EntryPrinter:
         """
         if meta is None:
             return
+        if prefix is None:
+            prefix = self.prefix
         for key, value in sorted(meta.items()):
             if key not in self.META_IGNORE:
                 value_str = None
@@ -139,8 +147,16 @@ class EntryPrinter:
                     value_str = 'TRUE' if value else 'FALSE'
                 elif isinstance(value, (dict, inventory.Inventory)):
                     pass # Ignore dicts, don't print them out.
+                elif value is None:
+                    value_str = ''  # Render null metadata as empty, on purpose.
                 else:
-                    raise ValueError("Unexpected value: '{!r}'".format(value))
+                    if self.stringify_invalid_types:
+                        # This is only intended to be used during development,
+                        # when debugging for custom values of data types
+                        # attached directly and not coming from the parser.
+                        value_str = str(value)
+                    else:
+                        raise ValueError("Unexpected value: '{!r}'".format(value))
                 if value_str is not None:
                     oss.write("{}{}: {}\n".format(prefix, key, value_str))
 
@@ -181,8 +197,8 @@ class EntryPrinter:
                                if self.render_weight
                                else False)
         if non_trivial_balance:
-            fmt = "  {{:{0}}}  {{:{1}}}  ; {{:{2}}}\n".format(
-                width_account, width_position, width_weight).format
+            fmt = "{0}{{:{1}}}  {{:{2}}}  ; {{:{3}}}\n".format(
+                self.prefix, width_account, width_position, width_weight).format
             for posting, account, position_str, weight_str in zip(entry.postings,
                                                                   strs_account,
                                                                   strs_position,
@@ -193,7 +209,8 @@ class EntryPrinter:
                 if posting.meta:
                     self.write_metadata(posting.meta, oss, '    ')
         else:
-            fmt_str = "  {{:{0}}}  {{:{1}}}\n".format(width_account, max(1, width_position))
+            fmt_str = "{0}{{:{1}}}  {{:{2}}}\n".format(
+                self.prefix, width_account, max(1, width_position))
             fmt = fmt_str.format
             for posting, account, position_str in zip(entry.postings,
                                                       strs_account,
@@ -241,7 +258,8 @@ class EntryPrinter:
         # method rendering a transaction attempts to align the posting strings
         # together.
         flag_account, position_str, weight_str = self.render_posting_strings(posting)
-        oss.write('  {:64} {} ; {}\n'.format(flag_account,
+        oss.write('{}{:64} {} ; {}\n'.format(self.prefix,
+                                             flag_account,
                                              position_str,
                                              weight_str).rstrip())
         if posting.meta:
@@ -326,7 +344,7 @@ class EntryPrinter:
         self.write_metadata(entry.meta, oss)
 
 
-def format_entry(entry, dcontext=None, render_weights=False):
+def format_entry(entry, dcontext=None, render_weights=False, prefix=None):
     """Format an entry into a string in the same input syntax the parser accepts.
 
     Args:
@@ -336,7 +354,7 @@ def format_entry(entry, dcontext=None, render_weights=False):
     Returns:
       A string, the formatted entry.
     """
-    return EntryPrinter(dcontext, render_weights)(entry)
+    return EntryPrinter(dcontext, render_weights, prefix=prefix)(entry)
 
 
 def print_entry(entry, dcontext=None, render_weights=False, file=None):
@@ -348,11 +366,16 @@ def print_entry(entry, dcontext=None, render_weights=False, file=None):
       render_weights: A boolean, true to render the weights for debugging.
       file: An optional file object to write the entries to.
     """
-    output = file or codecs.getwriter("utf-8")(sys.stdout.buffer)
+    output = file or (codecs.getwriter("utf-8")(sys.stdout.buffer)
+                      if hasattr(sys.stdout, 'buffer') else
+                      sys.stdout)
     output.write(format_entry(entry, dcontext, render_weights))
     output.write('\n')
 
 
+# TODO(blais): Change this to a function which accepts the same optional
+# arguments as the printer object. Isolate the spacer/segmentation algorithm to
+# its own function.
 def print_entries(entries, dcontext=None, render_weights=False, file=None, prefix=None):
     """A convenience function that prints a list of entries to a file.
 
@@ -384,6 +407,8 @@ def print_entries(entries, dcontext=None, render_weights=False, file=None, prefi
         output.write(string)
 
 
+# TODO(blais): Rename to format_source() to be consistent, better:
+# format_location().
 def render_source(meta):
     """Render the source for errors in a way that it will be both detected by
     Emacs and align and rendered nicely.
