@@ -143,6 +143,7 @@ class TestEntryPrinter(cmptest.TestCase):
 
     @loader.load_doc()
     def test_Document(self, entries, errors, __):
+        # pylint: disable=line-too-long
         """
         option "plugin_processing_mode" "raw"
         2014-06-01 open Assets:Account1
@@ -208,9 +209,8 @@ class TestEntryPrinter(cmptest.TestCase):
     def test_metadata(self):
         meta = data.new_metadata('beancount/core/testing.beancount', 12345)
         meta['something'] = r'a"\c'
-        pr = printer.EntryPrinter()
         oss = io.StringIO()
-        pr.write_metadata(meta, oss)
+        printer.EntryPrinter().write_metadata(meta, oss)
         self.assertEqual('  something: "a\\"\\\\c"\n', oss.getvalue())
 
 
@@ -534,3 +534,24 @@ class TestPrinterMisc(test_utils.TestCase):
         txn = errors[0].entry
         oss = io.StringIO()
         printer.print_entry(txn, file=oss)
+
+    def test_render_meta_with_None(self):
+        # Issue 378.
+        input_string = textwrap.dedent("""
+
+          2019-01-01 open Assets:A
+          2019-01-01 open Assets:B
+
+          2019-02-28 txn "Test"
+            Assets:A                       10.00 USD
+            Assets:B                      -10.00 USD
+            foo:
+
+        """)
+        entries, errors, options_map = loader.load_string(input_string)
+        self.assertFalse(errors)
+        self.assertIs(entries[-1].postings[-1].meta['foo'], None)
+
+
+if __name__ == '__main__':
+    unittest.main()
