@@ -12,8 +12,8 @@ import datetime
 import decimal
 import re
 import textwrap
+from decimal import Decimal
 
-from beancount.core.number import Decimal
 from beancount.core.number import ZERO
 from beancount.core.data import Transaction
 from beancount.core.compare import hash_entry
@@ -305,7 +305,7 @@ class OpenDate(query_compile.EvalFunction):
 
     def __call__(self, context):
         args = self.eval_args(context)
-        open_entry, close_entry = context.open_close_map[args[0]]
+        open_entry, _ = context.open_close_map[args[0]]
         return open_entry.date if open_entry else None
 
 class CloseDate(query_compile.EvalFunction):
@@ -317,7 +317,7 @@ class CloseDate(query_compile.EvalFunction):
 
     def __call__(self, context):
         args = self.eval_args(context)
-        close_entry, close_entry = context.open_close_map[args[0]]
+        _, close_entry = context.open_close_map[args[0]]
         return close_entry.date if close_entry else None
 
 class Meta(query_compile.EvalFunction):
@@ -418,7 +418,9 @@ class CurrencyMeta(query_compile.EvalFunction):
 
     def __call__(self, context):
         args = self.eval_args(context)
-        commodity_entry = context.commodity_map[args[0]]
+        commodity_entry = context.commodity_map.get(args[0], None)
+        if commodity_entry is None:
+            return {}
         return commodity_entry.meta
 
 
@@ -799,7 +801,7 @@ class Date(query_compile.EvalFunction):
     __intypes__ = [int, int, int]
 
     def __init__(self, operands):
-        super().__init__(operands, inventory.Inventory)
+        super().__init__(operands, datetime.date)
 
     def __call__(self, context):
         args = self.eval_args(context)
@@ -811,7 +813,7 @@ class ParseDate(query_compile.EvalFunction):
     __intypes__ = [str]
 
     def __init__(self, operands):
-        super().__init__(operands, inventory.Inventory)
+        super().__init__(operands, datetime.date)
 
     def __call__(self, context):
         args = self.eval_args(context)
@@ -1057,11 +1059,12 @@ class Min(query_compile.EvalAggregator):
         self.handle = allocator.allocate()
 
     def initialize(self, store):
-        store[self.handle] = self.dtype()
+        store[self.handle] = None
 
     def update(self, store, context):
         value = self.eval_args(context)[0]
-        if value < store[self.handle]:
+        cur_value = store[self.handle]
+        if cur_value is None or value < cur_value:
             store[self.handle] = value
 
     def __call__(self, context):
@@ -1078,11 +1081,13 @@ class Max(query_compile.EvalAggregator):
         self.handle = allocator.allocate()
 
     def initialize(self, store):
-        store[self.handle] = self.dtype()
+        store[self.handle] = None
 
     def update(self, store, context):
         value = self.eval_args(context)[0]
-        if value > store[self.handle]:
+        value = self.eval_args(context)[0]
+        cur_value = store[self.handle]
+        if cur_value is None or value > cur_value:
             store[self.handle] = value
 
     def __call__(self, context):
@@ -1242,7 +1247,7 @@ class DescriptionEntryColumn(query_compile.EvalColumn):
     def __call__(self, context):
         return (' | '.join(filter(None, [context.entry.payee,
                                          context.entry.narration]))
-                if isinstance(entry, Transaction)
+                if isinstance(context.entry, Transaction)
                 else None)
 
 
@@ -1524,7 +1529,7 @@ class AccountColumn(query_compile.EvalColumn):
         return context.posting.account
 
 class OtherAccountsColumn(query_compile.EvalColumn):
-    "The list of other accounts in the transcation, excluding that of this posting."
+    "The list of other accounts in the transaction, excluding that of this posting."
     __intypes__ = [data.Posting]
 
     def __init__(self):
