@@ -21,6 +21,7 @@ import operator
 import random
 import re
 import sys
+import string
 import textwrap
 
 from dateutil import rrule
@@ -44,7 +45,7 @@ from beancount.scripts import format
 from beancount.core import getters
 from beancount.utils import misc_utils
 from beancount.utils import date_utils
-from beancount.utils import version
+from beancount.parser import version
 from beancount import loader
 
 
@@ -191,7 +192,6 @@ def parse(input_string, **replacements):
       A list of directive objects.
     """
     if replacements:
-        import string
         class IgnoreFormatter(string.Formatter):
             def check_unused_args(self, used_args, args, kwargs):
                 pass
@@ -251,7 +251,7 @@ def date_random_seq(date_begin, date_end, days_min, days_max):
 
 
 def delay_dates(date_iter, delay_days_min, delay_days_max):
-    """Delay the dates from the given iterator by some uniformly dranw number of days.
+    """Delay the dates from the given iterator by some uniformly drawn number of days.
 
     Args:
       date_iter: An iterator of datetime.date instances.
@@ -536,7 +536,7 @@ def generate_tax_preamble(date_birth):
     """, **locals())
 
 def generate_tax_accounts(year, date_max):
-    """Generate accounts and contributino directives for a particular tax year.
+    """Generate accounts and contribution directives for a particular tax year.
 
     Args:
       year: An integer, the year we're to generate this for.
@@ -755,20 +755,21 @@ def generate_taxable_investment(date_begin, date_end, entries, price_map, stocks
       A list of directives.
     """
     account = 'Assets:CC:Investment'
+    income = 'Income:CC:Investment'
     account_cash = join(account, 'Cash')
-    account_gains = 'Income:CC:Investment:Gains'
-    account_dividends = 'Income:CC:Investment:Dividends'
+    account_gains = '{income}:PnL'.format(income=income)
+    dividends = 'Dividend'
     accounts_stocks = ['Assets:CC:Investment:{}'.format(commodity)
                        for commodity in stocks]
 
     open_entries = parse("""
       {date_begin} open {account}:Cash    CCY
       {date_begin} open {account_gains}    CCY
-      {date_begin} open {account_dividends}    CCY
     """, **locals())
     for stock in stocks:
         open_entries.extend(parse("""
           {date_begin} open {account}:{stock} {stock}
+          {date_begin} open {income}:{stock}:{dividends}    CCY
         """, **locals()))
 
     # Figure out dates at which dividends should be distributed, near the end of
@@ -814,12 +815,13 @@ def generate_taxable_investment(date_begin, date_end, entries, price_map, stocks
             portfolio_cost = total.reduce(convert.get_cost).get_currency_units('CCY').number
             amount_cash = (frac_dividend * portfolio_cost).quantize(D('0.01'))
             amount_cash_neg = -amount_cash
-            dividend = parse("""
+            stock = random.choice(stocks)
+            cash_dividend = parse("""
               {next_dividend_date} * "Dividends on portfolio"
                 {account}:Cash        {amount_cash:.2f} CCY
-                {account_dividends}   {amount_cash_neg:.2f} CCY
+                {income}:{stock}:{dividends}   {amount_cash_neg:.2f} CCY
             """, **locals())[0]
-            new_entries.append(dividend)
+            new_entries.append(cash_dividend)
 
             # Advance the next dividend date.
             try:
@@ -1359,7 +1361,7 @@ def price_series(start, mu, sigma):
 
 
 def generate_prices(date_begin, date_end, currencies, cost_currency):
-    """Generate weekly or monthyl price entries for the given currencies.
+    """Generate weekly or monthly price entries for the given currencies.
 
     Args:
       date_begin: The start date.
@@ -1560,7 +1562,7 @@ def write_example_file(date_birth, date_begin, date_end, reformat, file):
     destinations.extend(destinations)
     random.shuffle(destinations)
     for (date_trip_begin, date_trip_end), (destination_name, config) in zip(
-        compute_trip_dates(date_begin, date_end), destinations):
+            compute_trip_dates(date_begin, date_end), destinations):
 
         # Compute a suitable tag.
         tag = 'trip-{}-{}'.format(destination_name.lower().replace(' ', '-'),
